@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2022 Meltytech, LLC
+ * Copyright (c) 2013-2023 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,23 +15,30 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-function scrollIfNeeded(center) {
+function scrollIfNeeded(center, continouous) {
     var x = timeline.position * multitrack.scaleFactor;
     if (!tracksFlickable) return;
-    if (settings.timelineCenterPlayhead || center) {
+    if (center) {
         if (x > tracksFlickable.contentX + tracksFlickable.width * 0.5)
             tracksFlickable.contentX = x - tracksFlickable.width * 0.5;
         else if (x < tracksFlickable.width * 0.5)
             tracksFlickable.contentX = 0;
         else if (x < tracksFlickable.contentX + tracksFlickable.width * 0.5)
             tracksFlickable.contentX = x - tracksFlickable.width * 0.5;
-    } else {
-        if (x > tracksFlickable.contentX + tracksFlickable.width - 50)
-            tracksFlickable.contentX = x - tracksFlickable.width + 50;
-        else if (x < 50)
-            tracksFlickable.contentX = 0;
-        else if (x < tracksFlickable.contentX + 50)
-            tracksFlickable.contentX = x - 50;
+    } else if (tracksContainer.width > tracksFlickable.width) {
+        if (continouous) {
+            if (x > tracksFlickable.contentX + tracksFlickable.width - 50)
+                tracksFlickable.contentX = x - tracksFlickable.width + 50;
+            else if (x < 50)
+                tracksFlickable.contentX = 0;
+            else if (x < tracksFlickable.contentX + 50)
+                tracksFlickable.contentX = x - 50;
+        } else {
+            // paginated
+            var leftLimit = tracksFlickable.contentX + 50
+            var pageCount = Math.floor((x - leftLimit)/(tracksFlickable.width - 100))
+            tracksFlickable.contentX = Math.max(tracksFlickable.contentX + pageCount*(tracksFlickable.width - 100), 0);
+        }
     }
 }
 
@@ -99,10 +106,7 @@ function dropped() {
 
 function acceptDrop(xml) {
     var position = Math.round((dropTarget.x + tracksFlickable.contentX - headerWidth) / multitrack.scaleFactor)
-    if (settings.timelineRipple)
-        timeline.insert(timeline.currentTrack, position, xml, false)
-    else
-        timeline.overwrite(timeline.currentTrack, position, xml, false)
+    timeline.handleDrop(timeline.currentTrack, position, xml)
 }
 
 function trackHeight(isAudio) {
@@ -127,7 +131,7 @@ function onMouseWheel(wheel) {
             adjustZoom(wheel.angleDelta.y / 2000, wheel.x)
         }
         if (wheel.modifiers & Qt.ShiftModifier) {
-            n = (application.OS === 'OS X')? wheel.angleDelta.x : wheel.angleDelta.y
+            n = (application.OS === 'macOS')? wheel.angleDelta.x : wheel.angleDelta.y
             multitrack.trackHeight = Math.max(10, multitrack.trackHeight + n / 25)
         }
     } else {
@@ -135,6 +139,10 @@ function onMouseWheel(wheel) {
         if ((wheel.pixelDelta.x || wheel.pixelDelta.y) && wheel.modifiers === Qt.NoModifier) {
             var x = wheel.pixelDelta.x
             var y = wheel.pixelDelta.y
+            if (application.OS !== 'Windows' && !x && y) {
+                x = y;
+                y = 0;
+            }
             // Track pads provide both horizontal and vertical.
             if (!y || Math.abs(x) > 2)
                 tracksFlickable.contentX = clamp(tracksFlickable.contentX - x, 0, scrollMax().x)
@@ -142,7 +150,7 @@ function onMouseWheel(wheel) {
         } else {
             // Vertical only mouse wheel requires modifier for vertical scroll.
             if (wheel.modifiers === Qt.AltModifier) {
-                n = Math.round((application.OS === 'OS X'? wheel.angleDelta.y : wheel.angleDelta.x) / 2)
+                n = Math.round((application.OS === 'macOS'? wheel.angleDelta.y : wheel.angleDelta.x) / 2)
                 tracksFlickable.contentY = clamp(tracksFlickable.contentY - n, 0, scrollMax().y)
             } else {
                 n = Math.round(wheel.angleDelta.y / 2)
